//------------------------------------------------------------------
/*
	Controls.c
	
	Routines to perform automatic control of APD bias voltage and the time expansion
	
*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <asio.h>

//------------------------------------------------------------------
// Protoypes of external routines

//------------------------------------------------------------------ START
void APD_Control(void)
/*
	Routine to control the noise on the APD by adjusting the bias voltage.
	A noise setpoint is entered by the user. If it is zero the control loop is off.
	
	The noise value is controlled between [setpoint] and [2 x setpoint].
	The gap between the two setpoints provides hysterysis.
	
	The bias voltage is only changed when the results index is 1. This guves stability to the loop.
	
	INPUTS:		None
	RETURNS:	None	
*/
  {
  if((Index == 1) && (con_noise != 0)) 		// Check that the control loop is active
    {																			// It it is...
    if(noise_int > (2 * con_noise))				// If the noise is too high
      APD_Voltage--;											// Drop the APD bias voltage
    else if(noise_int < con_noise)				// If the noise is too low
      APD_Voltage++;											// Increase the APD bias voltage
			
    if (APD_Voltage <50)									// Check for the lower control limit
      APD_Voltage = 50;
    else if(APD_Voltage > 200)						// Check for the upper control limit
      APD_Voltage = 200;
    }
  }
//------------------------------------------------------------------
void Tex_Control(void)
/*
	Routine to control the amount of time expansion by adjusting the frequency of the second crystal.
	A resolution setpoint is entered by the user. If it is zero then the control loop is off.
	The resolution setpoint is specified in centimeters.
	If the setpoint is positive then forward time expansion results.
	If the setpoint is negative then reverse time expansion results.
	
	The actual control is done using the span because it has higher precision.
	Control takes place between [setpoint] and [setpoint + 10%].
	The gap between the two setpoints provides hysterysis.
	
	The crystal frequency is only changed when the results index is 1. This gives stability to the loop.
	
	INPUTS:		None
	RETURNS:	None	
*/
  {
	int con_span;
	
	if(con_resolution > 9)															// Limit the range of control to +9/-9 cm
		con_resolution = 9;
	if(con_resolution < -9)
		con_resolution = -9;
		
	if((Index == 1) & (con_resolution != 0))						// Check that the control loop is active
		{																									// If it is...
		
																											// Test for a request to change in time expansion direction
		if((con_resolution < 0) & (direction == FORWARD))	// If current time expansion is forward but the request is for reverse then
			Second_Xtal = 64;																// Jump to reverse time expansion
		if((con_resolution > 0) & (direction == REVERSE))	// If current time expansion is reverse but the request is for forward then
			Second_Xtal = 192;															// Jump to forward time expansion
			
																											// All controlling is done using the span value
		con_span = (int)(range * 100 / con_resolution);		// Calculate the control span using the current range and control resolution
		if(con_span < 0)
			con_span = 0 - con_span;												// Include the sign to acount for forward and reverse time expansion

    if(span_int > (con_span + con_span / 10))					// If the actual span > control span + 10% (both signed integers) then
			{
			if (con_resolution > 0)													// If time expansion is forward
      	Second_Xtal++;																// Increase the crystal frequency
			else
				Second_Xtal--;																// For reverse time expansion, decrease the crystal frequency
			}
			
    else if(span_int < con_span)											// If the actual span < control span (both signals signed integers) then
			{
      if (con_resolution > 0)													// If time expansion is forward
      	Second_Xtal--;																// Decrease the crystal frequency
			else
				Second_Xtal++;																// For reverse time expansion, increase the crystal frequency
			}
    if (Second_Xtal <10)															// Keep the crystal DAC voltage between 10 and 245 counts
      Second_Xtal = 10;																// This limits the control range to between +9cm and -9cm
    if(Second_Xtal > 245)
      Second_Xtal = 245;
		}	
	}	
//------------------------------------------------------------------ END